import static java.lang.Math.min;
import static java.util.Arrays.deepToString;
import static java.util.Arrays.fill;

import java.io.*;
import java.util.*;

/**
 * Solution, that uses Aho-Corasick. Time complexity is O(|T| + Sum(|A_i|) + K),
 * where K is the number of entries of strings from A to T. So as we have no
 * suffix entry in the set A, number of entries is |T|
 * 
 * @author Niyaz Nigmatullin
 * 
 */
public class cards_nn_aho {

	static final int ALPHABET = 26;

	static void solve() throws Exception {
		String t = next();
		int n = nextInt();
		String[] a = new String[n];
		for (int i = 0; i < n; i++) {
			a[i] = next();
		}
		Answer ans = answer(t, a);
		out.println(ans == null ? "No solution" : ans);
	}

	static Answer answer(String t, String[] a) {
		{
			int[] p = getPrefixFunction(t);
			for (String e : a) {
				if (containsSubstring(e, t, p)) {
					return new Answer(1, e);
				}
			}
		}
		Entry[][] ends = getWhatEndsHere(t, a);
		int[] dp = new int[t.length() + 1];
		Entry[] lastEntryUsed = new Entry[t.length() + 1];
		fill(dp, Integer.MAX_VALUE);
		dp[0] = 0;
		for (int i = 1; i <= t.length(); i++) {
			for (Entry e : ends[i - 1]) {
				int j = e.len;
				if (dp[i - j] == Integer.MAX_VALUE) {
					continue;
				}
				if (dp[i] > dp[i - j] + 1) {
					lastEntryUsed[i] = e;
					dp[i] = dp[i - j] + 1;
				}
			}
		}
		if (dp[t.length()] == Integer.MAX_VALUE) {
			return null;
		}
		List<String> words = new ArrayList<String>();
		for (int pos = t.length(); pos > 0; pos -= lastEntryUsed[pos].len) {
			words.add(lastEntryUsed[pos].string);
		}
		Collections.reverse(words);
		StringBuilder ret = new StringBuilder();
		for (String e : words) {
			ret.append(e);
		}
		return new Answer(dp[t.length()], ret.toString());
	}

	/**
	 * Time complexity is O(K + |T| + Sum(|A_i|)), where K is number of entries
	 * of strings from set to the text. Of course, K = O(|T| * |A|).
	 * 
	 * @param t
	 * @param a
	 * @return
	 */
	static Entry[][] getWhatEndsHere(String t, String[] a) {
		AhoCorasick ac = new AhoCorasick(a);
		int pos = ac.root;
		List<Entry>[] ret = new List[t.length()];
		for (int i = 0; i < ret.length; i++) {
			ret[i] = new ArrayList<Entry>(1);
		}
		List<String> temp = new ArrayList<String>();
		for (int i = 0; i < t.length(); i++) {
			pos = ac.go(pos, t.charAt(i));
			ac.addAllEntries(pos, temp);
			for (String j : temp) {
				ret[i].add(new Entry(j, j.length()));
			}
			temp.clear();
		}
		int[][] prefixes = new int[a.length][];
		for (int i = 0; i < a.length; i++) {
			prefixes[i] = getPrefixFunction(a[i]);
		}
		int[] pT = getPrefixFunction(t);
		// Here we add all entries as suffix of A_i equals to prefix of T
		for (int i = 0; i < a.length; i++) {
			int k = -1;
			int[] p = pT;
			String b = a[i];
			for (int j = 0; j < b.length(); j++) {
				while (k > -1
						&& (k + 1 >= t.length() || b.charAt(j) != t
								.charAt(k + 1))) {
					k = p[k];
				}
				if (k + 1 < t.length() && b.charAt(j) == t.charAt(k + 1)) {
					k++;
				}
			}
			while (k > -1) {
				ret[k].add(new Entry(b, k + 1));
				k = p[k];
			}
		}
		// Here we add all entries as suffix of T equals to prefix of A_i
		for (int i = 0; i < a.length; i++) {
			int k = -1;
			int[] p = prefixes[i];
			String b = a[i];
			for (int j = Math.max(0, t.length() - b.length()); j < t.length(); j++) {
				while (k > -1
						&& (k + 1 >= b.length() || b.charAt(k + 1) != t
								.charAt(j))) {
					k = p[k];
				}
				if (k + 1 < b.length() && t.charAt(j) == b.charAt(k + 1)) {
					k++;
				}
			}
			while (k > -1) {
				ret[t.length() - 1].add(new Entry(b, k + 1));
				k = p[k];
			}
		}
		Entry[][] ans = new Entry[ret.length][];
		for (int i = 0; i < ans.length; i++) {
			ans[i] = ret[i].toArray(new Entry[ret[i].size()]);
		}
		return ans;
	}

	static int[] getPrefixFunction(String text) {
		int[] p = new int[text.length()];
		int k = -1;
		p[0] = -1;
		for (int i = 1; i < p.length; i++) {
			while (k > -1 && text.charAt(k + 1) != text.charAt(i)) {
				k = p[k];
			}
			if (text.charAt(k + 1) == text.charAt(i)) {
				k++;
			}
			p[i] = k;
		}
		return p;
	}

	static boolean containsSubstring(String text, String word, int[] p) {
		int k = -1;
		for (int i = 0; i < text.length(); i++) {
			char c = text.charAt(i);
			while (k > -1
					&& (k + 1 >= word.length() || word.charAt(k + 1) != c)) {
				k = p[k];
			}
			if (k + 1 < word.length() && word.charAt(k + 1) == c) {
				k++;
			}
			if (k == word.length() - 1) {
				return true;
			}
		}
		return false;
	}

	static class AhoCorasick {
		int[][] link;
		int[] sufLink;
		String[] terminates;
		int free;
		final int root = 0;

		public AhoCorasick(String[] a) {
			int sumLen = 1;
			for (String e : a) {
				sumLen += e.length();
			}
			link = new int[ALPHABET][sumLen];
			sufLink = new int[sumLen];
			terminates = new String[sumLen];
			free = 1;
			for (int[] d : link) {
				fill(d, -1);
			}
			fill(sufLink, -1);
			for (String e : a) {
				addWord(e);
			}
			makeSufLinks();
		}

		void addAllEntries(int pos, List<String> temp) {
			if (terminates[pos] == null) {
				pos = sufLink[pos];
			}
			for (; pos > 0; pos = sufLink[pos]) {
				temp.add(terminates[pos]);
			}
		}

		int go(int pos, char c) {
			return link[c - 'a'][pos];
		}

		void addWord(String word) {
			int pos = 0;
			for (int i = 0; i < word.length(); i++) {
				int c = word.charAt(i) - 'a';
				if (link[c][pos] < 0) {
					link[c][pos] = free++;
				}
				pos = link[c][pos];
			}
			terminates[pos] = word;
		}

		void makeSufLinks() {
			int[] q = new int[sufLink.length];
			int head = 0;
			int tail = 1;
			sufLink[0] = 0;
			while (head < tail) {
				int v = q[head++];
				for (int i = 0; i < link.length; i++) {
					int u = link[i][v];
					int w = v == root ? root : link[i][sufLink[v]];
					if (u < 0) {
						link[i][v] = w;
					} else {
						q[tail++] = u;
						sufLink[u] = w;
					}
				}
			}
			for (int i = 0; i < head; i++) {
				int j = q[i];
				if (terminates[sufLink[j]] == null) {
					sufLink[j] = sufLink[sufLink[j]];
				}
			}
		}
	}

	static class Answer {
		int countWords;
		String text;

		public Answer(int countWords, String text) {
			this.countWords = countWords;
			this.text = text;
		}

		@Override
		public String toString() {
			return countWords + "\n" + text;
		}
	}

	static class Entry {
		String string;
		int len;

		public Entry(String string, int len) {
			this.string = string;
			this.len = len;
		}

		@Override
		public String toString() {
			return string + " with length " + len;
		}

	}

	static BufferedReader br;
	static StringTokenizer st;
	static PrintWriter out;

	static void debug(Object... a) {
		System.err.println(deepToString(a));
	}

	static int nextInt() throws IOException {
		return Integer.parseInt(next());
	}

	static String next() throws IOException {
		while (st == null || !st.hasMoreTokens()) {
			String line = br.readLine();
			if (line == null) {
				return null;
			}
			st = new StringTokenizer(line);
		}
		return st.nextToken();
	}

	static long nextLong() throws IOException {
		return Long.parseLong(next());
	}

	static double nextDouble() throws IOException {
		return Double.parseDouble(next());
	}

	public static void main(String[] args) {
		try {
			File file = new File("cards.in");
			InputStream input = System.in;
			OutputStream output = System.out;
			if (file.canRead()) {
				input = (new FileInputStream(file));
				output = (new PrintStream("cards.out"));
			}
			br = new BufferedReader(new InputStreamReader(input));
			out = new PrintWriter(output);
			solve();
			out.close();
			br.close();
		} catch (Throwable e) {
			e.printStackTrace();
			System.exit(1);
		}
	}
}
